@@ -49,65 +49,16 @@ G_BEGIN_DECLS
  * In the table below this is shown for the cases that a tag exists in the list
  * (A) or does not exists (!A) and combinations thereof.
  *
- * <table frame="all" colsep="1" rowsep="1">
- *   <title>merge mode</title>
- *   <tgroup cols='5' align='left'>
- *     <thead>
- *       <row>
- *         <entry>merge mode</entry>
- *         <entry>A + B</entry>
- *         <entry>A + !B</entry>
- *         <entry>!A + B</entry>
- *         <entry>!A + !B</entry>
- *       </row>
- *     </thead>
- *     <tbody>
- *       <row>
- *         <entry>REPLACE_ALL</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>REPLACE</entry>
- *         <entry>B</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>APPEND</entry>
- *         <entry>A, B</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>PREPEND</entry>
- *         <entry>B, A</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>KEEP</entry>
- *         <entry>A</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>KEEP_ALL</entry>
- *         <entry>A</entry>
- *         <entry>A</entry>
- *         <entry>-</entry>
- *         <entry>-</entry>
- *       </row>
- *     </tbody>
- *   </tgroup>
- * </table>
+ * | merge mode  | A + B | A + !B | !A + B | !A + !B |
+ * | ----------- | ----- | ------ | ------ | ------- |
+ * | REPLACE_ALL | B     | ø      | B      | ø       |
+ * | REPLACE     | B     | A      | B      | ø       |
+ * | APPEND      | A, B  | A      | B      | ø       |
+ * | PREPEND     | B, A  | A      | B      | ø       |
+ * | KEEP        | A     | A      | B      | ø       |
+ * | KEEP_ALL    | A     | A      | ø      | ø       |
  */
+
 typedef enum {
   GST_TAG_MERGE_UNDEFINED,
   GST_TAG_MERGE_REPLACE_ALL,
@@ -154,7 +105,7 @@ struct _GstTagList {
   GstMiniObject mini_object;
 };
 
-GST_EXPORT GType _gst_tag_list_type;
+GST_API GType _gst_tag_list_type;
 
 #define GST_TAG_LIST(x)       ((GstTagList *) (x))
 #define GST_TYPE_TAG_LIST     (_gst_tag_list_type)
@@ -183,16 +134,19 @@ typedef void (*GstTagForeachFunc) (const GstTagList * list,
  */
 typedef void (* GstTagMergeFunc) (GValue *dest, const GValue *src);
 
+GST_API
 GType        gst_tag_list_get_type (void);
 
 /* tag registration */
+
+GST_API
 void         gst_tag_register      (const gchar     * name,
                                     GstTagFlag        flag,
                                     GType             type,
                                     const gchar     * nick,
                                     const gchar     * blurb,
                                     GstTagMergeFunc   func);
-
+GST_API
 void         gst_tag_register_static (const gchar   * name,
                                       GstTagFlag      flag,
                                       GType           type,
@@ -201,15 +155,31 @@ void         gst_tag_register_static (const gchar   * name,
                                       GstTagMergeFunc func);
 
 /* some default merging functions */
+
+GST_API
 void      gst_tag_merge_use_first          (GValue * dest, const GValue * src);
+
+GST_API
 void      gst_tag_merge_strings_with_comma (GValue * dest, const GValue * src);
 
 /* basic tag support */
+
+GST_API
 gboolean               gst_tag_exists          (const gchar * tag);
+
+GST_API
 GType                  gst_tag_get_type        (const gchar * tag);
+
+GST_API
 const gchar *          gst_tag_get_nick        (const gchar * tag);
+
+GST_API
 const gchar *          gst_tag_get_description (const gchar * tag);
+
+GST_API
 GstTagFlag             gst_tag_get_flag        (const gchar * tag);
+
+GST_API
 gboolean               gst_tag_is_fixed        (const gchar * tag);
 
 /* tag lists */
@@ -227,219 +197,266 @@ typedef enum {
   GST_TAG_SCOPE_GLOBAL
 } GstTagScope;
 
+GST_API
 GstTagList * gst_tag_list_new_empty         (void) G_GNUC_MALLOC;
-GstTagList * gst_tag_list_new               (const gchar * tag, ...) G_GNUC_MALLOC;
+
+GST_API
+GstTagList * gst_tag_list_new               (const gchar * tag, ...) G_GNUC_NULL_TERMINATED G_GNUC_MALLOC;
+
+GST_API
 GstTagList * gst_tag_list_new_valist        (va_list var_args) G_GNUC_MALLOC;
 
+GST_API
 void         gst_tag_list_set_scope         (GstTagList * list, GstTagScope scope);
+
+GST_API
 GstTagScope  gst_tag_list_get_scope         (const GstTagList * list);
 
+GST_API
 gchar      * gst_tag_list_to_string         (const GstTagList * list) G_GNUC_MALLOC;
+
+GST_API
 GstTagList * gst_tag_list_new_from_string   (const gchar      * str) G_GNUC_MALLOC;
 
+GST_API
 gint         gst_tag_list_n_tags            (const GstTagList * list);
+
+GST_API
 const gchar* gst_tag_list_nth_tag_name      (const GstTagList * list, guint index);
+
+GST_API
 gboolean     gst_tag_list_is_empty          (const GstTagList * list);
+
+GST_API
 gboolean     gst_tag_list_is_equal          (const GstTagList * list1,
                                              const GstTagList * list2);
+GST_API
 void         gst_tag_list_insert            (GstTagList       * into,
                                              const GstTagList * from,
                                              GstTagMergeMode    mode);
+GST_API
 GstTagList * gst_tag_list_merge             (const GstTagList * list1,
                                              const GstTagList * list2,
                                              GstTagMergeMode    mode) G_GNUC_MALLOC;
+GST_API
 guint        gst_tag_list_get_tag_size      (const GstTagList * list,
                                              const gchar      * tag);
+GST_API
 void         gst_tag_list_add               (GstTagList       * list,
                                              GstTagMergeMode    mode,
                                              const gchar      * tag,
                                              ...) G_GNUC_NULL_TERMINATED;
+GST_API
 void         gst_tag_list_add_values        (GstTagList       * list,
                                              GstTagMergeMode    mode,
                                              const gchar      * tag,
                                              ...) G_GNUC_NULL_TERMINATED;
+GST_API
 void         gst_tag_list_add_valist        (GstTagList       * list,
                                              GstTagMergeMode    mode,
                                              const gchar      * tag,
                                              va_list        var_args);
+GST_API
 void         gst_tag_list_add_valist_values (GstTagList       * list,
                                              GstTagMergeMode    mode,
                                              const gchar      * tag,
                                              va_list            var_args);
+GST_API
 void         gst_tag_list_add_value         (GstTagList       * list,
                                              GstTagMergeMode    mode,
                                              const gchar      * tag,
                                              const GValue     * value);
+GST_API
 void         gst_tag_list_remove_tag        (GstTagList       * list,
                                              const gchar      * tag);
+GST_API
 void         gst_tag_list_foreach           (const GstTagList * list,
                                              GstTagForeachFunc  func,
                                              gpointer           user_data);
-
+GST_API
 const GValue *
              gst_tag_list_get_value_index   (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index);
+GST_API
 gboolean     gst_tag_list_copy_value        (GValue           * dest,
                                              const GstTagList * list,
                                              const gchar      * tag);
 
 /* simplifications (FIXME: do we want them?) */
+
+GST_API
 gboolean     gst_tag_list_get_boolean       (const GstTagList * list,
                                              const gchar      * tag,
                                              gboolean         * value);
+GST_API
 gboolean     gst_tag_list_get_boolean_index (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              gboolean         * value);
+GST_API
 gboolean     gst_tag_list_get_int           (const GstTagList * list,
                                              const gchar      * tag,
                                              gint             * value);
+GST_API
 gboolean     gst_tag_list_get_int_index     (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              gint             * value);
+GST_API
 gboolean     gst_tag_list_get_uint          (const GstTagList * list,
                                              const gchar      * tag,
                                              guint            * value);
+GST_API
 gboolean     gst_tag_list_get_uint_index    (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              guint            * value);
+GST_API
 gboolean     gst_tag_list_get_int64         (const GstTagList * list,
                                              const gchar      * tag,
                                              gint64           * value);
+GST_API
 gboolean     gst_tag_list_get_int64_index   (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              gint64           * value);
+GST_API
 gboolean     gst_tag_list_get_uint64        (const GstTagList * list,
                                              const gchar      * tag,
                                              guint64          * value);
+GST_API
 gboolean     gst_tag_list_get_uint64_index  (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              guint64          * value);
+GST_API
 gboolean     gst_tag_list_get_float         (const GstTagList * list,
                                              const gchar      * tag,
                                              gfloat           * value);
+GST_API
 gboolean     gst_tag_list_get_float_index   (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              gfloat           * value);
+GST_API
 gboolean     gst_tag_list_get_double        (const GstTagList * list,
                                              const gchar      * tag,
                                              gdouble          * value);
+GST_API
 gboolean     gst_tag_list_get_double_index  (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              gdouble          * value);
+GST_API
 gboolean     gst_tag_list_get_string        (const GstTagList * list,
                                              const gchar      * tag,
                                              gchar           ** value);
+GST_API
 gboolean     gst_tag_list_get_string_index  (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              gchar           ** value);
+GST_API
 gboolean     gst_tag_list_peek_string_index (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              const gchar     ** value);
+GST_API
 gboolean     gst_tag_list_get_pointer       (const GstTagList * list,
                                              const gchar      * tag,
                                              gpointer         * value);
+GST_API
 gboolean     gst_tag_list_get_pointer_index (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              gpointer         * value);
+GST_API
 gboolean     gst_tag_list_get_date          (const GstTagList * list,
                                              const gchar      * tag,
                                              GDate           ** value);
+GST_API
 gboolean     gst_tag_list_get_date_index    (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              GDate           ** value);
+GST_API
 gboolean     gst_tag_list_get_date_time     (const GstTagList * list,
                                              const gchar      * tag,
                                              GstDateTime     ** value);
+GST_API
 gboolean     gst_tag_list_get_date_time_index (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              GstDateTime     ** value);
+GST_API
 gboolean     gst_tag_list_get_sample        (const GstTagList * list,
                                              const gchar      * tag,
                                              GstSample       ** sample);
+GST_API
 gboolean     gst_tag_list_get_sample_index  (const GstTagList * list,
                                              const gchar      * tag,
                                              guint              index,
                                              GstSample       ** sample);
 
+#ifndef GST_DISABLE_MINIOBJECT_INLINE_FUNCTIONS
 /* refcounting */
-/**
- * gst_tag_list_ref:
- * @taglist: the #GstTagList to reference
- *
- * Add a reference to a #GstTagList mini object.
- *
- * From this point on, until the caller calls gst_tag_list_unref() or
- * gst_tag_list_make_writable(), it is guaranteed that the taglist object will
- * not change. To use a #GstTagList object, you must always have a refcount on
- * it -- either the one made implicitly by e.g. gst_tag_list_new(), or via
- * taking one explicitly with this function.
- *
- * Returns: the same #GstTagList mini object.
- */
-#ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC GstTagList * gst_tag_list_ref (GstTagList * taglist);
-#endif
-
 static inline GstTagList *
 gst_tag_list_ref (GstTagList * taglist)
 {
   return (GstTagList *) gst_mini_object_ref (GST_MINI_OBJECT_CAST (taglist));
 }
 
-/**
- * gst_tag_list_unref:
- * @taglist: a #GstTagList.
- *
- * Unref a #GstTagList, and and free all its memory when the refcount reaches 0.
- */
-#ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC void gst_tag_list_unref (GstTagList * taglist);
-#endif
-
 static inline void
 gst_tag_list_unref (GstTagList * taglist)
 {
   gst_mini_object_unref (GST_MINI_OBJECT_CAST (taglist));
 }
 
-/**
- * gst_tag_list_copy:
- * @taglist: a #GstTagList.
- *
- * Creates a new #GstTagList as a copy of the old @taglist. The new taglist
- * will have a refcount of 1, owned by the caller, and will be writable as
- * a result.
- *
- * Note that this function is the semantic equivalent of a gst_tag_list_ref()
- * followed by a gst_tag_list_make_writable(). If you only want to hold on to a
- * reference to the data, you should use gst_tag_list_ref().
- *
- * When you are finished with the taglist, call gst_tag_list_unref() on it.
- *
- * Returns: the new #GstTagList
- */
-#ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC GstTagList * gst_tag_list_copy (const GstTagList * taglist);
-#endif
-
-static inline GstTagList *
-gst_tag_list_copy (const GstTagList * taglist)
+static inline void
+gst_clear_tag_list (GstTagList ** taglist_ptr)
 {
-  return GST_TAG_LIST (gst_mini_object_copy (GST_MINI_OBJECT_CAST (taglist)));
+  gst_clear_mini_object ((GstMiniObject **) taglist_ptr);
 }
+#else /* GST_DISABLE_MINIOBJECT_INLINE_FUNCTIONS */
+GST_API
+GstTagList *  gst_tag_list_ref   (GstTagList * taglist);
+
+GST_API
+void          gst_tag_list_unref (GstTagList * taglist);
+
+GST_API
+void          gst_clear_tag_list (GstTagList ** taglist_ptr);
+#endif /* GST_DISABLE_MINIOBJECT_INLINE_FUNCTIONS */
+
+GST_API
+GstTagList* gst_tag_list_copy(const GstTagList* taglist);
+
+#define gst_tag_list_copy(taglist) GST_TAG_LIST (gst_mini_object_copy (GST_MINI_OBJECT_CAST (taglist)))
+
+#ifndef GST_DISABLE_MINIOBJECT_INLINE_FUNCTIONS
+static inline gboolean
+gst_tag_list_replace (GstTagList **old_taglist, GstTagList *new_taglist)
+{
+    return gst_mini_object_replace ((GstMiniObject **) old_taglist,
+        (GstMiniObject *) new_taglist);
+}
+
+static inline gboolean
+gst_tag_list_take (GstTagList **old_taglist, GstTagList *new_taglist)
+{
+  return gst_mini_object_take ((GstMiniObject **) old_taglist,
+      (GstMiniObject *) new_taglist);
+}
+#else /* GST_DISABLE_MINIOBJECT_INLINE_FUNCTIONS */
+GST_API
+gboolean  gst_tag_list_replace (GstTagList ** old_taglist,
+                                GstTagList * new_taglist);
+
+GST_API
+gboolean  gst_tag_list_take    (GstTagList ** old_taglist,
+                                GstTagList * new_taglist);
+#endif
 
 /**
  * gst_tag_list_is_writable:
@@ -541,6 +558,14 @@ gst_tag_list_copy (const GstTagList * taglist)
  * person(s) who composed the recording (string)
  */
 #define GST_TAG_COMPOSER               "composer"
+/**
+ * GST_TAG_CONDUCTOR:
+ *
+ * conductor/performer refinement (string)
+ *
+ * Since: 1.8
+ */
+#define GST_TAG_CONDUCTOR               "conductor"
 /**
  * GST_TAG_DATE:
  *
@@ -801,7 +826,7 @@ gst_tag_list_copy (const GstTagList * taglist)
  * ISO-639-2 or ISO-639-1 code for the language the content is in (string)
  *
  * There is utility API in libgsttag in gst-plugins-base to obtain a translated
- * language name from the language code: gst_tag_get_language_name()
+ * language name from the language code: `gst_tag_get_language_name()`
  */
 #define GST_TAG_LANGUAGE_CODE          "language-code"
 /**
@@ -818,7 +843,7 @@ gst_tag_list_copy (const GstTagList * taglist)
  * GST_TAG_IMAGE:
  *
  * image (sample) (sample taglist should specify the content type and preferably
- * also set "image-type" field as #GstTagImageType)
+ * also set "image-type" field as `GstTagImageType`)
  */
 #define GST_TAG_IMAGE                  "image"
 /**
@@ -1065,13 +1090,44 @@ gst_tag_list_copy (const GstTagList * taglist)
 /**
  * GST_TAG_MIDI_BASE_NOTE:
  *
- * <ulink url="http://en.wikipedia.org/wiki/Note#Note_designation_in_accordance_with_octave_name">Midi note number</ulink>
+ * [Midi note number](http://en.wikipedia.org/wiki/Note#Note_designation_in_accordance_with_octave_name)
  * of the audio track. This is useful for sample instruments and in particular
  * for multi-samples.
  *
  * Since: 1.4
  */
 #define GST_TAG_MIDI_BASE_NOTE                    "midi-base-note"
+/**
+ * GST_TAG_PRIVATE_DATA:
+ *
+ * Any private data that may be contained in tags (sample).
+ *
+ * It is represented by #GstSample in which #GstBuffer contains the
+ * binary data and the sample's info #GstStructure may contain any
+ * extra information that identifies the origin or meaning of the data.
+ *
+ * Private frames in ID3v2 tags ('PRIV' frames) will be represented
+ * using this tag, in which case the GstStructure will be named
+ * "ID3PrivateFrame" and contain a field named "owner" of type string
+ * which contains the owner-identification string from the tag.
+ *
+ * Since: 1.8
+ */
+#define GST_TAG_PRIVATE_DATA                         "private-data"
+
+/**
+ * GST_TAG_CONTAINER_SPECIFIC_TRACK_ID:
+ *
+ * Unique identifier for the audio, video or text track this tag is associated
+ * with. The mappings for several container formats are defined in the [Sourcing
+ * In-band Media Resource Tracks from Media Containers into HTML
+ * specification](https://dev.w3.org/html5/html-sourcing-inband-tracks/).
+ *
+ * Since: 1.24
+ */
+#define GST_TAG_CONTAINER_SPECIFIC_TRACK_ID "container-specific-track-id"
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstTagList, gst_tag_list_unref)
 
 G_END_DECLS
 
